﻿
using Microsoft.VisualBasic;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Windows.Forms;

namespace Theremino_HAL
{
    public partial class Form2 : Form
    {
        Class_DataScope Scope;

        private void Form2_Load(object sender, System.EventArgs e)
        {
            btn_SetZero.ClickButtonArea += btn_SetZero_ClickButtonArea;
            btn_ShowRawCount.ClickButtonArea += btn_ShowRawCount_ClickButtonArea;

            Scope = new Class_DataScope(pbox_ScrollingScope);
            SetScaleLabels();
        }

        private void Form2_VisibleChanged(object sender, System.EventArgs e)
        {
            StartTimer();
        }

        internal void StopTimer()
        {
            Timer_60Hz.Stop();
        }

        internal void StartTimer()
        {
            if (Visible & WindowState != FormWindowState.Minimized)
            {
                Timer_60Hz.Interval = 16;
                Timer_60Hz.Start();
            }
            else
            {
                Timer_60Hz.Stop();
            }
        }

        private void Form2_FormClosing(object sender, System.Windows.Forms.FormClosingEventArgs e)
        {
            GroupBox1.BackColor = Color.LightGoldenrodYellow;
            this.Hide();
            e.Cancel = true;
        }

        private void Form2_LocationChanged(object sender, System.EventArgs e)
        {
            if (!Module_SaveLoad.EventsAreEnabled) return;

            Module_SaveLoad.LimitFormPosition(this);
        }


        private void Timer_60Hz_Tick(System.Object sender, System.EventArgs e)
        {
            if (!Module_SaveLoad.EventsAreEnabled) return;
            ShowPinDetails();
        }

        private Pin SelectedPin;
        private Pin AlternatePin;
        private Int32 SelectedPinListLine; 
        private Int32 AlternatePinListLine;
       
        internal void RestoreSelectedPins()
        {
            SelectedPin = null;
            AlternatePin = null;
            pbox_ScrollingScope.Visible = true;
            LabelPin1.Text = "- - -";
            lbl_Details1.Text = "";
            Module_FillBars.FillPictureBox(pbox_Details1, 0, Color.Orange, Color.AliceBlue);
            LabelPin2.Text = "- - -";
            lbl_Details2.Text = "";
            Module_FillBars.FillPictureBox(pbox_Details2, 0, Color.Orange, Color.AliceBlue);
            // ------------------------------------------------------ recover selected pins from list lines
            SelectedPin = ThereminoSystem.FindPinByListLine(SelectedPinListLine);
            AlternatePin = ThereminoSystem.FindPinByListLine(AlternatePinListLine);
            InitializePinDetails();
        }


        internal void SetPinDetails()
        {
            Pin p = ThereminoSystem.GetSelectedPin();
            Int32 l = ThereminoSystem.GetSelectedListLine();
            //
            if (p == null) return;
            //
            if (object.ReferenceEquals(p, SelectedPin))
            {
                AlternatePin = null;
                AlternatePinListLine = 0;
                SelectedPin = p;
                SelectedPinListLine = l;
            }
            else
            {
                AlternatePin = SelectedPin;
                AlternatePinListLine = SelectedPinListLine;
                SelectedPin = p;
                SelectedPinListLine = l;
            }
            InitializePinDetails();
        }


        internal void InitializePinDetails()
        {
            if (SelectedPin != null)
            {
                LabelPin1.Text = ComposePinString(SelectedPin);
            }
            if (AlternatePin != null)
            {
                LabelPin2.Text = ComposePinString(AlternatePin);
            }
            else
            {
                LabelPin2.Text = "- - -";
                lbl_Details2.Text = "";
                Module_FillBars.FillPictureBox(pbox_Details2, 0, Color.Orange, Color.AliceBlue);
            }
            //
            // ThereminoSystem.TheSystem_CalibrateZero();
            SetScaleLabels();
        }

        private void ShowPinDetails()
        {
            ShowDetails();
            UpdateScrollingScope();
            if (btn_BeepOnChange.Checked) BeepOnChange();
        }

        private string ComposePinString(Pin p)
        {
            return "M:" + (p.MasterId + 1).ToString() + " S:" + (p.SlaveId + 1).ToString() + " Pin:" + (p.PinId + 1).ToString();
        }

        private Int32 ShowDetailsDivisor;
        private void ShowDetails()
        {
            ShowDetailsDivisor += 1;
            if (ShowDetailsDivisor >= 2) // 2 = 30 executions per second (timer 60Hz divided by 2)
            {
                ShowDetailsDivisor = 0;
                if (SelectedPin != null)
                {
                    lbl_Details1.Text = SelectedPin.GetValueString();
                    Module_FillBars.FillPictureBox(pbox_Details1, SelectedPin.Value_Normalized, Color.Orange, Color.AliceBlue);
                }
                if (AlternatePin != null)
                {
                    lbl_Details2.Text = AlternatePin.GetValueString();
                    Module_FillBars.FillPictureBox(pbox_Details2, AlternatePin.Value_Normalized, Color.Orange, Color.AliceBlue);
                }
            }
        }

        static float static_BeepOnChange_oldc = 0;
        private void BeepOnChange()
        {
            if (SelectedPin != null)
            {
                float c = SelectedPin.Value;
                if (c != static_BeepOnChange_oldc && c != 0 && !Single.IsNaN(c))
                {
                    static_BeepOnChange_oldc = c;
                    //Debug.Print(SelectedPin.Value.ToString());
                    //Interaction.Beep();
                    Module_Utils.Beep_RepetitionLimited(150);
                }
            }
        }

        private void btn_SetZero_ClickButtonArea(System.Object Sender, System.EventArgs e)
        {
            ThereminoSystem.TheSystem_CalibrateZero();
            SetScaleLabels();
        }

        private void btn_ShowRawCount_ClickButtonArea(System.Object Sender, System.EventArgs e)
        {
            ThereminoSystem.TheSystem_CalibrateZero();
            SetScaleLabels();
        }

        private void UpdateScrollingScope()
        {
            if (!pbox_ScrollingScope.Visible) return;
            Single v1 = -999;
            Single v2 = -999;
            Single zoom = tkbar_VerticalScaleZoom.Value;
            if (btn_ShowRawCount.Checked)
            {
                if (SelectedPin != null)
                {
                    v1 = (SelectedPin.Value_RawUinteger_Zero - (int)SelectedPin.Value_RawUinteger) * zoom / 100.0F;
                }
                if (AlternatePin != null)
                {
                    v2 = (AlternatePin.Value_RawUinteger_Zero - (int)AlternatePin.Value_RawUinteger) * zoom / 100.0F;
                }
            }
            else
            {
                if (zoom == 1.0f)
                {
                    if (SelectedPin != null)
                    {
                        v1 = pbox_ScrollingScope.Height * 0.49F * (0.5F - SelectedPin.Value_Normalized);
                    }
                    if (AlternatePin != null)
                    {
                        v2 = pbox_ScrollingScope.Height * 0.49F * (0.5F - AlternatePin.Value_Normalized);
                    }
                }
                else
                {
                    if (SelectedPin != null)
                    {
                        v1 = pbox_ScrollingScope.Height * 0.49F * (SelectedPin.Value_Normalized_Zero - SelectedPin.Value_Normalized) * zoom;
                    }
                    if (AlternatePin != null)
                    {
                        v2 = pbox_ScrollingScope.Height * 0.49F * (AlternatePin.Value_Normalized_Zero - AlternatePin.Value_Normalized) * zoom;
                    }
                }
            }
            Scope.Display(v1, v2, tkbar_ScrollSpeed.Value);
        }

         private void SetScaleLabels()
         {
            Label_Scale3.Text = "";
            Label_Scale2.Text = "";
            Label_Scale1.Text = "";
            Label_Scale3b.Text = "";
            Label_Scale2b.Text = "";
            Label_Scale1b.Text = "";
            double zoom  = tkbar_VerticalScaleZoom.Value;
            if (btn_ShowRawCount.Checked)
            {
                if (SelectedPin != null)
                {
                    Label_Scale3.Text = (SelectedPin.Value_RawUinteger_Zero + 6800 / zoom).ToString("0");
                    Label_Scale2.Text = SelectedPin.Value_RawUinteger_Zero.ToString("0");
                    Label_Scale1.Text = (SelectedPin.Value_RawUinteger_Zero - 6800 / zoom).ToString("0");
                }
                if (AlternatePin != null)
                {
                    Label_Scale3b.Text = (AlternatePin.Value_RawUinteger_Zero + 6800 / zoom).ToString("0");
                    Label_Scale2b.Text = AlternatePin.Value_RawUinteger_Zero.ToString("0");
                    Label_Scale1b.Text = (AlternatePin.Value_RawUinteger_Zero - 6800 / zoom).ToString("0");
                }
             }
             else
             {
                 if (zoom == 1.0f)
                 {
                    Label_Scale3.Text = "1000";
                    Label_Scale2.Text = "500";
                    Label_Scale1.Text = "0";
                 }
                 else
                 {
                     if (SelectedPin != null)
                     {
                        float zv = SelectedPin.Value_Normalized_Zero * 1000.0F;
                        Label_Scale3.Text = (zv + 500 / zoom).ToString("0");
                        Label_Scale2.Text = zv.ToString("0");
                        Label_Scale1.Text = (zv - 500 / zoom).ToString("0");
                     }
                     if (AlternatePin != null)
                     {
                        float zv = AlternatePin.Value_Normalized_Zero * 1000.0F;
                        Label_Scale3b.Text = (zv + 500 / zoom).ToString("0");
                        Label_Scale2b.Text = zv.ToString("0");
                        Label_Scale1b.Text = (zv - 500 / zoom).ToString("0");
                     }
                 }
             }
         }

         private void tkbar_VerticalScaleZoom_Scroll(object sender, EventArgs e)
         {
             SetScaleLabels();
         }




        public Form2()
        {
            LocationChanged += Form2_LocationChanged;
            FormClosing += Form2_FormClosing;
            SizeChanged += Form2_VisibleChanged;
            VisibleChanged += Form2_VisibleChanged;
            Load += Form2_Load;
            InitializeComponent();
        }
    }
}
